<?php

namespace Stabuses\Stabuses\Policies;

use Litepie\User\Contracts\UserPolicy;
use Stabuses\Stabuses\Models\Stabuses;

class StabusesPolicy
{

    /**
     * Determine if the given user can view the stabuses.
     *
     * @param UserPolicy $user
     * @param Stabuses $stabuses
     *
     * @return bool
     */
    public function view(UserPolicy $user, Stabuses $stabuses)
    {
        if ($user->canDo('stabuses.stabuses.view') && $user->isAdmin()) {
            return true;
        }

        return $stabuses->user_id == user_id() && $stabuses->user_type == user_type();
    }

    /**
     * Determine if the given user can create a stabuses.
     *
     * @param UserPolicy $user
     * @param Stabuses $stabuses
     *
     * @return bool
     */
    public function create(UserPolicy $user)
    {
        return  $user->canDo('stabuses.stabuses.create');
    }

    /**
     * Determine if the given user can update the given stabuses.
     *
     * @param UserPolicy $user
     * @param Stabuses $stabuses
     *
     * @return bool
     */
    public function update(UserPolicy $user, Stabuses $stabuses)
    {
        if ($user->canDo('stabuses.stabuses.edit') && $user->isAdmin()) {
            return true;
        }

        return $stabuses->user_id == user_id() && $stabuses->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given stabuses.
     *
     * @param UserPolicy $user
     * @param Stabuses $stabuses
     *
     * @return bool
     */
    public function destroy(UserPolicy $user, Stabuses $stabuses)
    {
        return $stabuses->user_id == user_id() && $stabuses->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given stabuses.
     *
     * @param UserPolicy $user
     * @param Stabuses $stabuses
     *
     * @return bool
     */
    public function verify(UserPolicy $user, Stabuses $stabuses)
    {
        if ($user->canDo('stabuses.stabuses.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given stabuses.
     *
     * @param UserPolicy $user
     * @param Stabuses $stabuses
     *
     * @return bool
     */
    public function approve(UserPolicy $user, Stabuses $stabuses)
    {
        if ($user->canDo('stabuses.stabuses.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
