<?php

namespace Stabuses\Stabus\Providers;

use Illuminate\Support\ServiceProvider;

class StabusServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'stabus');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'stabus');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerStabus();
        $this->registerFacade();
        $this->registerBindings();
        //$this->registerCommands();
    }


    /**
     * Register the application bindings.
     *
     * @return void
     */
    protected function registerStabus()
    {
        $this->app->bind('stabus', function($app) {
            return new Stabus($app);
        });
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->booting(function()
        {
            $loader = \Illuminate\Foundation\AliasLoader::getInstance();
            $loader->alias('Stabus', 'Lavalite\Stabus\Facades\Stabus');
        });
    }

    /**
     * Register bindings for the provider.
     *
     * @return void
     */
    public function registerBindings() {
        // Bind facade
        $this->app->bind('stabuses.stabus', function ($app) {
            return $this->app->make('Stabuses\Stabus\Stabus');
        });

                // Bind Stabuses to repository
        $this->app->bind(
            'Stabuses\Stabus\Interfaces\StabusesRepositoryInterface',
            \Stabuses\Stabus\Repositories\Eloquent\StabusesRepository::class
        );

        $this->app->register(\Stabuses\Stabus\Providers\AuthServiceProvider::class);
        
        $this->app->register(\Stabuses\Stabus\Providers\RouteServiceProvider::class);
            }

    /**
     * Merges user's and stabus's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/stabus.php', 'stabus'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                Commands\MakeStabus::class,
            ]);
        }
    }
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['stabuses.stabus'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/config.php' => config_path('stabuses/stabus.php')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/stabus')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/stabus')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
