<?php

namespace Stabuses\Stabuses\Providers;

use Illuminate\Support\ServiceProvider;

class StabusesServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'stabuses');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'stabuses');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerStabuses();
        $this->registerFacade();
        $this->registerBindings();
        //$this->registerCommands();
    }


    /**
     * Register the application bindings.
     *
     * @return void
     */
    protected function registerStabuses()
    {
        $this->app->bind('stabuses', function($app) {
            return new Stabuses($app);
        });
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->booting(function()
        {
            $loader = \Illuminate\Foundation\AliasLoader::getInstance();
            $loader->alias('Stabuses', 'Lavalite\Stabuses\Facades\Stabuses');
        });
    }

    /**
     * Register bindings for the provider.
     *
     * @return void
     */
    public function registerBindings() {
        // Bind facade
        $this->app->bind('stabuses.stabuses', function ($app) {
            return $this->app->make('Stabuses\Stabuses\Stabuses');
        });

                // Bind Stabuses to repository
        $this->app->bind(
            'Stabuses\Stabuses\Interfaces\StabusesRepositoryInterface',
            \Stabuses\Stabuses\Repositories\Eloquent\StabusesRepository::class
        );

        $this->app->register(\Stabuses\Stabuses\Providers\AuthServiceProvider::class);
        
        $this->app->register(\Stabuses\Stabuses\Providers\RouteServiceProvider::class);
            }

    /**
     * Merges user's and stabuses's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/stabuses.php', 'stabuses'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                Commands\MakeStabuses::class,
            ]);
        }
    }
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['stabuses.stabuses'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/config.php' => config_path('stabuses/stabuses.php')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/stabuses')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/stabuses')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
